//
//  AppDelegate.m
//  CryptingFunctionsExamlpe
//
//  Created by Florian on 05.03.13.
//  Copyright (c) 2013 whateva0x29a. All rights reserved.
//

#import "AppDelegate.h"
#import "CryptingFunctions.h"

@implementation AppDelegate

@synthesize window = _window;

@synthesize inputFileTextField = _inputFileTextField;
@synthesize outputFileTextField = _outputFileTextField;
@synthesize passwordTextField = _passwordTextField;

- (void)dealloc
{
    [super dealloc];
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
}

- (IBAction)browseInputFileButtonClicked:(id)sender 
{
    NSOpenPanel* openDlg = [NSOpenPanel openPanel];
    
    // Set selection properties
    [openDlg setCanChooseFiles:YES];
    [openDlg setCanChooseDirectories:NO];
    [openDlg setResolvesAliases:YES];
    [openDlg setAllowsMultipleSelection:NO];
    
    NSArray *files;
    NSString *filePath;
    
    // Display the dialog.  If the OK button was pressed,
    // process the files.
    if ( [openDlg runModal] == NSOKButton )
    {
        // Get an array containing the full filenames of all
        // files and directories selected.
        files = [openDlg URLs];
        
        // because you can actually only select one file we just take the first one
        filePath = [[files objectAtIndex:0] path];
        
        NSString *fileName = [NSString stringWithString:[[filePath lastPathComponent] stringByDeletingPathExtension]];
        
        self.inputFileTextField.stringValue = filePath;    
        self.outputFileTextField.stringValue = [NSString stringWithFormat:@"%@.crypted",fileName];
    }

}

- (IBAction)decryptButtonClicked:(id)sender 
{
    if (![self checkForCompleteness] )
        return;
    
    // these two lines are everything you need to decrypt the file
    NSError *error = NULL;
    
    // Note: e.g. if the user enters a wrong password we will have to load the whole file again and check it again (very time and cpu consuming), we'll leave it this way for now, but should change that
    NSData *fileData = [CryptingFunctions loadEncryptedFileAtPath:self.inputFileTextField.stringValue password:self.passwordTextField.stringValue userInfo:nil error:&error];
    
    // everything that follows is only error checking and show the user the error
    if ( error )
    {
        if (error.code == ERROR_CODE_WRONG_PASSWORD) 
        {
            NSAlert *newAlert = [NSAlert alertWithMessageText:error.localizedDescription defaultButton:@"OK" alternateButton:nil otherButton:nil informativeTextWithFormat:@"The password you entered to decrypt the file appears to be incorrect. Please try again."];
            
            [newAlert runModal];
        }
        
        if (error.code == ERROR_CODE_FILE_NOT_FOUND_OR_NOT_READABLE) 
        {
            NSAlert *newAlert = [NSAlert alertWithMessageText:error.localizedDescription defaultButton:@"OK" alternateButton:nil otherButton:nil informativeTextWithFormat:@"The input file could not be found or is not readable."];
            
            [newAlert runModal];
        }
        
        if(error.code == ERROR_CODE_CORRUPTED_FILE_FORMAT)
        {
            NSAlert *newAlert = [NSAlert alertWithMessageText:error.localizedDescription defaultButton:@"OK" alternateButton:nil otherButton:nil informativeTextWithFormat:@"The file could not be readed, because its file format is corrupted."];
            
            [newAlert runModal];
        }
        if(error.code == ERROR_CODE_DECRYPTING)
        {
            NSAlert *newAlert = [NSAlert alertWithMessageText:error.localizedDescription defaultButton:@"OK" alternateButton:nil otherButton:nil informativeTextWithFormat:@"The file couldn't be decrypted."];
            
            [newAlert runModal];
        }
        
        return;
    }
    
    // and we (try to) write the decrypted file to disk
    
    NSString *path = [NSString stringWithFormat:@"%@/%@", [self.inputFileTextField.stringValue stringByDeletingLastPathComponent], self.outputFileTextField.stringValue];
    
    BOOL success = [fileData writeToFile:path 
                              atomically:NO];
    
    // again check for errors
    if (!success) 
    {
        NSAlert *newAlert = [NSAlert alertWithMessageText:@"Could not write decrypted file." defaultButton:@"OK" alternateButton:nil otherButton:nil informativeTextWithFormat:@"Check if the output file already exist."];
        
        [newAlert runModal];
        return;
    }
    
    
    // tell the user that everyting went fine
    NSAlert *newAlert = [NSAlert alertWithMessageText:@"Succesfully decrypted file." defaultButton:@"OK" alternateButton:nil otherButton:nil informativeTextWithFormat:[NSString stringWithFormat:@"Wrote file to %@", path]];
    
    [newAlert runModal];
}

- (IBAction)encryptButtonClicked:(id)sender 
{
    if (![self checkForCompleteness] )
        return;
    
    
    if(![[NSFileManager defaultManager] isReadableFileAtPath:self.inputFileTextField.stringValue]) 
    {
        NSAlert *newAlert = [NSAlert alertWithMessageText:@"Input file not found or not readable." defaultButton:@"OK" alternateButton:nil otherButton:nil informativeTextWithFormat:@"The input file could not be found or is not readable."];
        [newAlert runModal];
        return;
    }
    
    NSData *fileData = [NSData dataWithContentsOfFile:self.inputFileTextField.stringValue];
   
    NSString *path = [NSString stringWithFormat:@"%@/%@", [self.inputFileTextField.stringValue stringByDeletingLastPathComponent], self.outputFileTextField.stringValue];
    
    NSError *error = NULL;
    
    [CryptingFunctions saveFileAtPath:path data:fileData password:self.passwordTextField.stringValue userInfo:nil error:&error];
    
    if (error)
    {
        if(error.code == ERROR_CODE_WRITING_FILE)
        {
            NSAlert *newAlert = [NSAlert alertWithMessageText:error.localizedDescription defaultButton:@"OK" alternateButton:nil otherButton:nil informativeTextWithFormat:@"Could not write output file. Check if the output file already exists."];
            [newAlert runModal];
        }
        if(error.code == ERROR_CODE_ENCRYPTING)
        {
            NSAlert *newAlert = [NSAlert alertWithMessageText:error.localizedDescription defaultButton:@"OK" alternateButton:nil otherButton:nil informativeTextWithFormat:@"The file couldn't be encrypted."];
            [newAlert runModal];
        }
        return;
    }
    
    // tell the user that everyting went fine
    NSAlert *newAlert = [NSAlert alertWithMessageText:@"Succesfully encrypted file." defaultButton:@"OK" alternateButton:nil otherButton:nil informativeTextWithFormat:[NSString stringWithFormat:@"Wrote file to %@", path]];
    
    [newAlert runModal];
    
}

-(BOOL)checkForCompleteness
{
    if ([self.inputFileTextField.stringValue isEqualToString:@""])
    {
        NSAlert *newAlert = [NSAlert alertWithMessageText:@"Please enter a input file." defaultButton:@"OK" alternateButton:nil otherButton:nil informativeTextWithFormat:@"You have to set an input file to decrypt or encrypt."];
        
        [newAlert runModal];
        return NO;
    }
    if([self.outputFileTextField.stringValue isEqualToString:@""])
    {
        NSAlert *newAlert = [NSAlert alertWithMessageText:@"Please enter a output filename." defaultButton:@"OK" alternateButton:nil otherButton:nil informativeTextWithFormat:@"You have to enter an output file name, where the encrypted or decrypted file will be saved."];
        
        [newAlert runModal];
        return NO;
    }
    if([self.passwordTextField.stringValue isEqualToString:@""])
    {
        NSAlert *newAlert = [NSAlert alertWithMessageText:@"Please enter a password." defaultButton:@"OK" alternateButton:nil otherButton:nil informativeTextWithFormat:@"You have to set a password to decrypt or encrypt a file."];
        
        [newAlert runModal];
        return NO;
    }
    return YES;
}

@end
